@@ -10,6 +10,11 @@ use chrono;
 #[cfg(feature = "approx")]
 use approx::ApproxEq;
 
+#[cfg(feature = "serde")]
+use serde::{Serialize, Deserialize, Serializer, Deserializer};
+#[cfg(feature = "serde")]
+use serde::de::{self, Visitor};
+
 use super::error;
 
 /// Number of nanoseconds in a second.
@@ -34,9 +39,9 @@ pub const SECS_PER_YEAR: f64 = SECS_PER_DAY * 365.0;
 /// via `float_duration_since` in either direction.
 pub trait TimePoint<Rhs = Self> {
     /// The type returned if there is an error computing the duration.
-    type Err;
+    type Error;
     /// The amount of time between two `TimePoint`s.
-    fn float_duration_since(self, rhs: Rhs) -> Result<FloatDuration, Self::Err>;
+    fn float_duration_since(self, rhs: Rhs) -> Result<FloatDuration, Self::Error>;
 }
 
 /// A time duration stored as a floating point quantity.
@@ -55,6 +60,13 @@ pub struct FloatDuration {
 
 
 impl FloatDuration {
+    /// Create a new `FloatDuration` representing a number of years.
+    ///
+    /// `float_duration` considers one year to be exactly 365 days, with
+    /// no consideration of leap years.
+    pub fn years(years: f64) -> FloatDuration {
+        FloatDuration { secs: years * SECS_PER_YEAR }
+    }
     /// Create a new `FloatDuration` representing a number of days.
     pub fn days(days: f64) -> FloatDuration {
         FloatDuration { secs: days * SECS_PER_DAY }
@@ -69,7 +81,7 @@ impl FloatDuration {
     }
     /// Create a new `FloatDuration` representing a number of seconds.
     pub fn seconds(secs: f64) -> FloatDuration {
-        FloatDuration { secs }
+        FloatDuration { secs: secs }
     }
     /// Create a new `FloatDuration` representing a number of milliseconds.
     pub fn milliseconds(millis: f64) -> FloatDuration {
@@ -84,6 +96,13 @@ impl FloatDuration {
         FloatDuration { secs: nanos / NANOS_PER_SEC }
     }
 
+    /// Return the total number of fractional years represented by the `FloatDuration`.
+    ///
+    /// `float_duration` considers one year to be exactly 365 days, with
+    /// no consideration of leap years.
+    pub fn as_years(&self) -> f64 {
+        self.secs / SECS_PER_YEAR
+    }
     /// Return the total number of fractional days represented by the `FloatDuration`.
     pub fn as_days(&self) -> f64 {
         self.secs / SECS_PER_DAY
@@ -205,9 +224,50 @@ impl FloatDuration {
     }
 }
 
+#[cfg(feature = "serde")]
+struct FloatDurationVisitor;
+
+// We want to serialize a `FloatDuration` as a single f64 instead of a struct.
+#[cfg(feature = "serde")]
+impl<'de> Visitor<'de> for FloatDurationVisitor {
+    type Value = FloatDuration;
+
+    fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
+        formatter.write_str("a floating-point value")
+    }
+    fn visit_f32<E>(self, value: f32) -> Result<FloatDuration, E>
+        where E: de::Error
+    {
+        Ok(FloatDuration::seconds(value as f64))
+    }
+    fn visit_f64<E>(self, value: f64) -> Result<FloatDuration, E>
+        where E: de::Error
+    {
+        Ok(FloatDuration::seconds(value))
+    }
+}
+
+#[cfg(feature = "serde")]
+impl Serialize for FloatDuration {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+        where S: Serializer
+    {
+        serializer.serialize_f64(self.secs)
+    }
+}
+
+#[cfg(feature = "serde")]
+impl<'de> Deserialize<'de> for FloatDuration {
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+        where D: Deserializer<'de>
+    {
+        deserializer.deserialize_f64(FloatDurationVisitor)
+    }
+}
+
 #[cfg(feature = "chrono")]
 impl<Tz: chrono::TimeZone> TimePoint for chrono::DateTime<Tz> {
-    type Err = ();
+    type Error = ();
     fn float_duration_since(self, since: chrono::DateTime<Tz>) -> Result<FloatDuration, ()> {
         let chrono_duration = self.signed_duration_since(since);
         Ok(FloatDuration::from_chrono(&chrono_duration))
@@ -215,7 +275,7 @@ impl<Tz: chrono::TimeZone> TimePoint for chrono::DateTime<Tz> {
 }
 #[cfg(feature = "chrono")]
 impl<Tz: chrono::TimeZone> TimePoint for chrono::Date<Tz> {
-    type Err = ();
+    type Error = ();
     fn float_duration_since(self, since: chrono::Date<Tz>) -> Result<FloatDuration, ()> {
         let chrono_duration = self.signed_duration_since(since);
         Ok(FloatDuration::from_chrono(&chrono_duration))
@@ -223,7 +283,7 @@ impl<Tz: chrono::TimeZone> TimePoint for chrono::Date<Tz> {
 }
 #[cfg(feature = "chrono")]
 impl TimePoint for chrono::NaiveDate {
-    type Err = ();
+    type Error = ();
     fn float_duration_since(self, since: chrono::NaiveDate) -> Result<FloatDuration, ()> {
         let chrono_duration = self.signed_duration_since(since);
         Ok(FloatDuration::from_chrono(&chrono_duration))
@@ -231,7 +291,7 @@ impl TimePoint for chrono::NaiveDate {
 }
 #[cfg(feature = "chrono")]
 impl TimePoint for chrono::NaiveTime {
-    type Err = ();
+    type Error = ();
     fn float_duration_since(self, since: chrono::NaiveTime) -> Result<FloatDuration, ()> {
         let chrono_duration = self.signed_duration_since(since);
         Ok(FloatDuration::from_chrono(&chrono_duration))
@@ -239,7 +299,7 @@ impl TimePoint for chrono::NaiveTime {
 }
 #[cfg(feature = "chrono")]
 impl TimePoint for chrono::NaiveDateTime {
-    type Err = ();
+    type Error = ();
     fn float_duration_since(self, since: chrono::NaiveDateTime) -> Result<FloatDuration, ()> {
         let chrono_duration = self.signed_duration_since(since);
         Ok(FloatDuration::from_chrono(&chrono_duration))
@@ -247,14 +307,14 @@ impl TimePoint for chrono::NaiveDateTime {
 }
 
 impl TimePoint for time::Instant {
-    type Err = ();
+    type Error = ();
     fn float_duration_since(self, since: time::Instant) -> Result<FloatDuration, ()> {
         let std_duration = self.duration_since(since);
         Ok(FloatDuration::from_std(std_duration))
     }
 }
 impl TimePoint for time::SystemTime {
-    type Err = time::SystemTimeError;
+    type Error = time::SystemTimeError;
     fn float_duration_since(self,
                             since: time::SystemTime)
                             -> Result<FloatDuration, time::SystemTimeError> {
@@ -265,7 +325,9 @@ impl TimePoint for time::SystemTime {
 
 impl fmt::Display for FloatDuration {
     fn fmt(&self, fmt: &mut fmt::Formatter) -> fmt::Result {
-        if self.secs > SECS_PER_DAY {
+        if self.secs > SECS_PER_YEAR {
+            write!(fmt, "{} years", self.as_years())
+        } else if self.secs > SECS_PER_DAY {
             write!(fmt, "{} days", self.as_days())
         } else if self.secs > SECS_PER_HOUR {
             write!(fmt, "{} hours", self.as_hours())
@@ -429,6 +491,8 @@ mod tests {
         assert_eq!(duration4.as_minutes(), -3.0);
         assert_eq!(duration4.as_hours(), -0.05);
         assert!(duration4.is_negative());
+
+        assert_eq!(FloatDuration::years(2.0), FloatDuration::days(365.0 * 2.0));
     }
 
     #[test]
@@ -488,14 +552,16 @@ mod tests {
         assert_eq!(format!("{}", FloatDuration::minutes(3.5)), "3.5 minutes");
         assert_eq!(format!("{}", FloatDuration::days(3.0) + FloatDuration::hours(12.0)),
                    "3.5 days");
+        assert_eq!(format!("{}", FloatDuration::seconds(12.7)), "12.7 seconds");
+        assert_eq!(format!("{}", FloatDuration::default()), "0 nanoseconds");
 
         assert_eq!(format!("{}", FloatDuration::microseconds(100.0)),
                    "100 microseconds");
         assert_eq!(format!("{}", FloatDuration::milliseconds(12.5)),
                    "12.5 milliseconds");
 
-        assert_eq!(format!("{}", FloatDuration::days(365.0) + FloatDuration::hours(6.0)),
-                   "365.25 days");
+        assert_eq!(format!("{}", FloatDuration::days(325.0) + FloatDuration::hours(6.0)),
+                   "325.25 days");
         assert_eq!(format!("{}",
                            FloatDuration::milliseconds(50.0) + FloatDuration::microseconds(500.0)),
                    "50.5 milliseconds");
@@ -503,6 +569,7 @@ mod tests {
         assert_eq!(format!("{}", FloatDuration::nanoseconds(25.25)),
                    "25.25 nanoseconds");
         assert_eq!(format!("{}", FloatDuration::minutes(90.0)), "1.5 hours");
+        assert_eq!(format!("{}", FloatDuration::years(2.5)), "2.5 years");
     }
 
     #[cfg(feature = "chrono")]
@@ -520,6 +587,8 @@ mod tests {
                    FloatDuration::zero());
         assert_eq!(FloatDuration::from_chrono(&chrono::Duration::hours(10000)),
                    FloatDuration::hours(10000.0));
+        assert_eq!(FloatDuration::from_chrono(&chrono::Duration::milliseconds(1i64 << 62)),
+                   FloatDuration::milliseconds((1i64 << 62) as f64));
 
         assert_eq!(FloatDuration::minutes(2.5).to_chrono().unwrap(),
                    chrono::Duration::seconds(150));
@@ -530,4 +599,39 @@ mod tests {
                    chrono::Duration::nanoseconds(-20));
 
     }
+
+    #[cfg(feature = "chrono")]
+    #[test]
+    fn test_chrono_timepoint() {
+        use chrono::{TimeZone, UTC, Local};
+
+        let date1 = UTC.ymd(2017, 5, 25).and_hms(10, 0, 0);
+        let date2 = UTC.ymd(2017, 5, 26).and_hms(12, 0, 0);
+
+        assert_eq!(date2.float_duration_since(date1).unwrap(),
+                   FloatDuration::days(1.0) + FloatDuration::hours(2.0));
+
+        let date3 = Local::now();
+        assert_eq!(date3.float_duration_since(date3).unwrap(),
+                   FloatDuration::zero());
+        assert!(date3.float_duration_since(Local::now()).unwrap() < FloatDuration::zero());
+    }
+
+    #[cfg(feature = "serde")]
+    #[test]
+    fn test_serde() {
+        use serde_test::{Token, assert_tokens};
+
+        let duration = FloatDuration::seconds(1.5);
+        assert_tokens(&duration, &[Token::F64(duration.as_seconds())]);
+
+        assert_tokens(&FloatDuration::zero(), &[Token::F64(0.0)]);
+
+        let duration2 = FloatDuration::hours(3.0);
+        assert_tokens(&duration2, &[Token::F64(3.0 * SECS_PER_HOUR)]);
+
+        let duration3 = FloatDuration::days(5.0) + FloatDuration::minutes(35.2) +
+                        FloatDuration::milliseconds(100.0);
+        assert_tokens(&duration3, &[Token::F64(duration3.as_seconds())]);
+    }
 }
